/*  mshell - A simple shell with statistics printing.
    Copyright (C) 2012 Lawrence Sebald

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted, provided that the above
    copyright notice and this permission notice appear in all copies.
 
    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
    WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
    MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
    SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
    WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
    OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
    CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/resource.h>
#include <readline/readline.h>

/* This function based on one in the readline documentation. */
char *rl_gets(const char *prompt) {
    char *rv = readline(prompt);

    if(rv && *rv)
        add_history(rv);
    return rv;
}

/* Split a string into a list of arguments, as is needed for execvp() */
char **split_args(char *input) {
    char *lasts, *tok, **rv = NULL;
    int nargs = 0;

    tok = strtok_r(input, " ", &lasts);
    while(tok) {
        /* Allocate space for the new argument */
        if(!(rv = (char **)realloc(rv, (nargs + 1) * sizeof(char *)))) {
            perror("realloc");
            exit(EXIT_FAILURE);
        }

        /* Put the new argument in the list and pull out the next one */
        rv[nargs++] = tok;
        tok = strtok_r(NULL, " ", &lasts);
    }

    /* Make space for the needed NULL at the end of the list */
    if(!(rv = (char **)realloc(rv, (nargs + 1) * sizeof(char *)))) {
        perror("realloc");
        exit(EXIT_FAILURE);
    }

    rv[nargs] = NULL;
    return rv;
}

int main(int argc, char *argv[]) {
    char *cmd, **args;
    pid_t pid;
    struct rusage ru;
    float timeused;
    int ncmds = 0;

    while((cmd = rl_gets("<mshell> $ "))) {
        if(!*cmd) continue;
        args = split_args(cmd);

        if(!(pid = fork())) {
            if(execvp(args[0], args)) {
                fprintf(stderr, "%s: %s\n", args[0], strerror(errno));
                exit(EXIT_FAILURE);
            }
        }
        else if(pid < 0) {
            perror("fork");
            exit(EXIT_FAILURE);
        }
        else {
            wait(NULL);
            if(!getrusage(RUSAGE_CHILDREN, &ru)) {
                timeused = (ru.ru_utime.tv_sec + ru.ru_stime.tv_sec) + 
                    (ru.ru_utime.tv_usec + ru.ru_stime.tv_usec) / 1000000.0f;
                printf("\n%d commands run\nTime used: %f\nMaximum Memory used: "
                       "%ld\n", ++ncmds, timeused, (long)ru.ru_maxrss);
            }

            free(args);
            free(cmd);
        }
    }

    printf("\n");
    return 0;
}
